#! /usr/bin/perl -w

use strict;

#########################################################
#
# (c) Serxio Barral - 2006
#
# changeagent.pl
#
# This perl program gathers change history info
#
# It reads two configuration files
#
# /etc/sumon/agent/changeagent.conf
#
#
#  This program is free software published under the
#  terms of the GNU General Public License.
#
#  For the entire license text please refer to
#  http://www.gnu.org/licenses/gpl.html
#
#
#  Serial: _182
#
#########################################################


# VAR
   my $confdir    = "/etc/sumon/agent";
   my $configfile = "$confdir/changeagent.conf"; 
   my $hostname   = "";
   my @files      = ();
   my @commands   = ();
   my @dirs       = ();
   my @dirsr      = ();
   my $insert     = 0;
   my $repository = "";
   my $file       = "";
   my $command    = "";
   my $dir        = "";
   my $tmpdir     = "/tmp";
   my $tmpfile    = $tmpdir."/chgmgmt.".$$.".tmp";
   my $timestamp  = time();
   my %findhash   = ();
   my $CHGMGMTDIR = ();
   my $argline    = "";
   my $fileread   = "";
   my $uname      = ""; 

   my %varhash    = (
                       "datadir"          => "/var/sumon/data",
                       "changeconfigfile" => "$confdir/changeinfo.conf",
                       "debug" => 0
                    );

   #my $BASEDIR    = "/home/remeye"; 
   #my $DATADIR    = $BASEDIR."/data"; 
   #my $CONFIGDIR  = "$BASEDIR"."/"."etc";
   #my $CONFIGFILE = "$CONFIGDIR"."/"."chgmgmtinfo.conf";

# ENDVAR


$uname      = `uname`;
$hostname   = `hostname`;
chomp ($hostname);
chomp ($uname);



# Var reading from config file

$argline = join (" ",@ARGV);

if ($argline =~ /-f\s*(\S+)\s*/)
{
   # There is an user specified  configuration file

   $configfile = $1;
}

if (-f $configfile) 
{
   open (CONFIGFILE,"< $configfile") || die "could not open config file \"$configfile\"\n";

   while (<CONFIGFILE>)
   {
      chomp($_);
   
      if (($_ !~ /^\s*#/) && ($_ =~ /^\s*(\S+)\s*=\s*(\S+)/))
      {
         my $readvar   = $1;
         my $readvalue = $2;
   
   
         if (defined ($varhash{$readvar}))
         {
            $varhash{$readvar} = $readvalue;
   
            if ($varhash{debug})
            {
               print "read from configfile $readvar $readvalue\n";
            }
         }
         else
         {
            print STDERR "variable not known to program: $readvar in config file, maybe an error\n";
         }
      }
   }
}

$CHGMGMTDIR = $varhash{"datadir"}."/chgmgmtinfo";
$repository = $CHGMGMTDIR."/chgmgmtinfo.".$hostname;

# Config file reading
##################################################
#COMMON:
#  FILE:/etc/hosts
#  FILE:/etc/services
#  FILE:/etc/inetd.conf
#  FILE:etc/inittab
#  COMMAND:"lsattr -E -l inet0"
#
#HOST:foomachine:
#  FILE:/Tivoli/tec/scripts/notify.pl
#
#HOST:barmachine:
#  FILE:/etc/oratab
##################################################

open (CONFIGFILE,"< $varhash{changeconfigfile}") || die "Error: cannot open configuration file \"$varhash{changeconfigfile}\"\n";

while (<CONFIGFILE>)
{
   # VAR
   my $line = $_;
   # ENDVAR

   chomp ($line);

   if  ($line =~ /^\s*COMMON:/i)
   {
      $insert = 1;
   }
   elsif ($line =~ /^\s*HOST:([^:]+):/i)
   {
       my $my_hostname = $1;

       if ($hostname =~ /$1/i)
       {
          if ($varhash{debug})
          {
             print "read HOST $my_hostname\n";
          }
          $insert = 1;
       }
       else
       {
          $insert = 0;
       }
   }
   elsif ($line =~ /^\s*UNAME:$uname:/i)
   {
      $insert = 1;
   }
   elsif ($line =~ /^\s*HOST:/)
   {
      $insert = 0;
   }
   elsif ($insert == 1)
   {
      if ($line =~ /^\s*FILE\s*:\s*(\/[^\s]*)/)
      {
         push (@files,$1);
      }
      elsif ($line =~ /^\s*COMMAND\s*:\s*\"([^#]*)\"/)
      {
         push (@commands,$1);
      } 
      elsif ($line =~ /^\s*DIR\s*:\s*(\/[^\s]*)/)
      {
         push (@dirs,$1);
      }
      elsif ($line =~ /^\s*DIRR\s*:\s*(\/[^\s]*)/)
      {
         push (@dirsr,$1);
      }
   }

}
close (CONFIGFILE);


(-d $repository) || system ("mkdir -p $repository");

(-d $repository) || die "Error: Cannot create directory \"$repository\"";

# just one find, better performance

open (FINDER,"find $repository -type f |") || die "cannot find in \"$repository\"\n";

while (<FINDER>)
{
   chomp ($_);

   if ($_ =~ /([^\/]+):(\d+):(\d+)/)
   {
      if (!defined $findhash{$1})
      {
         #VAR     
         my @findarray = ();
         #ENDVAR
         $findhash{$1} = \@findarray;
      }
      push (@{$findhash{$1}},$_);
   }
}
close (FINDER);

# @files has the files to be checked, @commands has the command lines...

foreach $file (@files)
{
   # VAR
   my $openstring = "";
   my $string     = "";
   my @statarray  = ();
   # ENDVAR

   if (-f $file)
   {
      @statarray = stat($file);
      $timestamp = $statarray[9];
   }
   else
   {
      $timestamp = 0;
   }

   ###print "$file $timestamp\n";

   $openstring = "< ".$file;
   $string     = &stringEncode ($file);

   &processItem($openstring,$string,"FILE");
}


foreach $command (@commands)
{
   # VAR 
   my $openstring = "";
   my $string     = "";
   # ENDVAR    

   $timestamp = time();

   #$openstring = $command." |";

   $openstring = $command." 2> /dev/null |";
   $string     = &stringEncode ($command);

   &processItem($openstring,$string,"COMMAND");
}

foreach $dir (@dirs)
{
   # VAR
   my $string             = "";
   my %directoryfileshash = ();
   my $file               = "";
   my $dirfile            = "";
   # ENDVAR


   $timestamp = time();

   # Checking first if there already exists a directory definition file 

   $string = &stringEncode ($dir);

   $dirfile = $repository . "/DIRECTORY:" . $string; 

   if (-f $dirfile)
   {
      # It does exist, including its entires in a file 
      open (DIRFILE,"< $dirfile") || die "Cannot open \"$dirfile\"\n";
      while (<DIRFILE>)
      {
         chomp($_);
         $directoryfileshash{$_} = 1;
      }
      close (DIRFILE);
   }
 
   # find /etc/* -prune -type f

   # finding

#   if (open (FINDDIR,"find $dir -prune -type f |"))
#   {
#      while (<FINDDIR>) 
#      {
#         chomp ($_);
#         $directoryfileshash{$_} = 1;
#      }
#      close (FINDDIR);
#   }

   if (opendir (FINDDIR,"$dir"))
   {
      while ($fileread = readdir(FINDDIR))
      {
         if (-f $dir."/".$fileread)
         {
            $directoryfileshash{$dir."/".$fileread} = 1;
         }
      }
      close (FINDDIR);
   }



   foreach $file (keys %directoryfileshash)
   {
         # VAR
         my $openstring = "";
         my $string     = "";
         my @statarray  = ();
         # ENDVAR
 
         if (-f $file)
         {
            @statarray = stat($file);
            $timestamp = $statarray[9];
         }
         else
         {
            $timestamp = 0;
         }

         $openstring = "< ".$file;
         $string     = &stringEncode ($file);

         &processItem($openstring,$string,"FILE");
   }

   # OK, we can save now the new list
   
   if (open (DIRFILE,"> $dirfile"))
   {

      foreach $file (keys %directoryfileshash)
      {
         print DIRFILE "$file\n";
      }

      close (DIRFILE);
   }
   
}


sub processItem
{
   # VAR
   my $openstring           = $_[0];
   my $encodedfilename      = $_[2].$_[1];
   my $filename             = "";
   my $checksum             = "";
   my $lastversionchecksum  = -1;
   my $lastversiontimestamp = -1;
   my $dirname              = "";
   # ENDVAR
  
   if (-f $tmpfile)
   {
      system ("rm $tmpfile");
   }

   ###print "Openstring $openstring  String $encodedfilename $filename\n";

   $dirname  = $repository."/".$encodedfilename;
   (-d $dirname) || system ("mkdir -p $dirname");


   open (TMPFILE,"> $tmpfile") || die "Cannot open \"$tmpfile\"\n";

   if (open (OPENSTRING,"$openstring"))
   {
      # Reading checksum
     
      while (<OPENSTRING>)
      {
         print TMPFILE $_;  
      }
      close (OPENSTRING);

      close (TMPFILE);

      open  (CKSUM,"/usr/bin/cksum $tmpfile |");
      while (<CKSUM>)
      {
         chomp($_);  
         if ($_ =~ /\s*(\d+)\s+(\d+)\s*/)  
         {
            if ($varhash{debug})
            {
               print "Checksum tmpfile $tmpfile $openstring $_\n";
            }
            $checksum = $1; 
         }
      }
      close (CKSUM);

    
   }  # End if open
   else
   {
      # if open fails we can asume that it does not exist... checksum = 0
      $checksum = 0;
   } 

   if (defined $findhash{$encodedfilename}) #looking up for some
   {
      my $item = "";

      foreach $item (@{$findhash{$encodedfilename}})
      {

         #          filename          TS    CKS
         if ($item =~ /$encodedfilename:(\d+):(\d+)/)
         {
            # Already have a version of this file
            # VAR
            my $filetimestamp = $1;
            my $filechecksum  = $2;
            # ENDVAR 
  
            if ($lastversiontimestamp < $filetimestamp)
            {
               $lastversiontimestamp = $filetimestamp;
               $lastversionchecksum  = $filechecksum;
            }
         }
      }
   } #end if defined

   $filename = $dirname . "/" . $encodedfilename . ":" . $timestamp . ":" . $checksum;

   if  ($checksum != $lastversionchecksum)
   {
      if ($checksum != 0)
      {
         system ("cp $tmpfile $filename");

         if ($varhash{debug}) 
         { 
            print "$checksum $lastversionchecksum\n";
         }
      }
      else
      {
         # cksum 0
         system ("touch $filename");
      }
   } # End if 

   if (-f $tmpfile)
   {
      unlink $tmpfile;
   }
}


sub stringEncode
{
   my $theURL = $_[0];
   $theURL =~ s/([\W])/"%" . uc(sprintf("%2.2x",ord($1)))/eg;
   return $theURL;
}

sub stringDecode
{
    my $theURL = $_[0];
    $theURL =~ tr/+/ /;
    $theURL =~ s/%([a-fA-F0-9]{2,2})/chr(hex($1))/eg;
    $theURL =~ s/<!--(.|\n)*-->//g;
    return $theURL;
}
