#! /usr/bin/perl -w

use strict;


#########################################################
#
# (c) Serxio Barral - 2005
#
# statsagent.pl
#
# This perl program gathers performance data
#
# It reads the configuration file
#
# /etc/sumon/agent/statsagent.conf
#
#
#  This program is free software published under the
#  terms of the GNU General Public License.
#
#  For the entire license text please refer to
#  http://www.gnu.org/licenses/gpl.html
#
#  Serial: _143
#
#########################################################


# VAR

my $confdir               = "/etc/sumon/agent";
my $bindir                = "/opt/sumon/agent/bin";
my $configfile            = "$confdir/statsagent.conf";

my %varhash = (
        "debug"          => 0,
        "outdir"         => "/var/sumon/stats/out",
        "fileprefix"     => "gstats",
        "master"         => "",
        "rsyncrepname"        => "sumonStats",
        "varrundir"      => "/var/sumon/run",
        "fifopostfix"    => "statsagent.fifo",
        "pidfilepostfix" => "statsagent.pid",
        "hastodiefile"   => "/var/sumon/run/statsagent.die",
        "LINUXmon"       => "lmon",
        "AIXmon"         => "nmon",
        "nmon"           => "/usr/sbin/nmon",
        "lmon"           => "$bindir/lmon.pl", 
        "test"           => 0,
        "maxsentlines"   => 100
              );
my $fifotime              = time % 10;
my $pidcheckrequired      = 0;
my $pid                   = "";
my $hostname              = "";
my $cpunumber             = 1;
my $hastodie              = 0;
my $cpu_min               = 0;
my $cpu_max               = 0;
my $portstring            = "";
my $rsync                 = "";
my %interfacehash         = ();
my $os                    = "LINUX";
my $argline               = "";
my $monitor               = "";
my $monitorpath           = "";
my $filename              = "";
my $sentlines             = 0;
my %states = (
             CLOSED      => 0,
             CLOSING     => 0,
             CLOSE_WAIT  => 0,
             ESTABLISHED => 0,
             FIN_WAIT_1  => 0,
             FIN_WAIT_2  => 0,
             LAST_ACK    => 0,
             LISTEN      => 0,
             SYN_RCVD    => 0,
             SYN_SENT    => 0,
             TIME_WAIT   => 0
          );

my @diskreadarray        = ();
my @diskwritearray       = ();
my %diskreadhash         = ();
my %diskwritehash        = ();
my %cdroms               = ();
my %linuxvaliddisks      = ();


my @physicalifacesread         = ("en0-read-KB/s" ,"en1-read-KB/s" ,"en2-read-KB/s" ,"en3-read-KB/s");
my @physicalifaceswrite        = ("en0-write-KB/s","en1-write-KB/s","en2-write-KB/s","en3-write-KB/s");

my @linuxphysicalifacesread    = ("eth0-read-KB/s","eth1-read-KB/s","eth2-read-KB/s","eth3-read-KB/s");
my @linuxphysicalifaceswrite   = ("eth0-write-KB/s","eth1-write-KB/s","eth2-write-KB/s","eth3-write-KB/s");

my @virtualifacesread          = ("lo0-read-KB/s");
my @virtualifaceswrite         = ("lo0-write-KB/s");

my @linuxvirtualifacesread     = ("lo-read-KB/s");
my @linuxvirtualifaceswrite    = ("lo-write-KB/s");



# Var reading from config file

$argline = join (" ",@ARGV);

if ($argline =~ /-f\s*(\S+)\s*/)
{
   # There is an use specified  configuration file

   $configfile =$1;
}

open (CONFIGFILE,"< $configfile") || die "could not open config file \"$configfile\"\n";

while (<CONFIGFILE>)
{
   chomp($_);

   if (($_ !~ /^\s*#/) && ($_ =~ /^\s*(\S+)\s*=\s*(\S+)/))
   {
      my $readvar   = $1; 
      my $readvalue = $2;
       
      
      if (defined ($varhash{$readvar}))
      {
         $varhash{$readvar} = $readvalue;
         if ($varhash{debug})
         {
            print "read from configfile $readvar $readvalue\n";
         }
      }
      else
      {
         print STDERR "variable not known to program: $readvar in config file, maybe an error\n";
      }
   }

}

my $fifo                  = $varhash{varrundir}."/".$varhash{fifopostfix}.$fifotime;
my $pidfile               = $varhash{varrundir}."/".$varhash{pidfilepostfix};

# ENDVAR


close (CONFIGFILE);


if ($varhash{"master"} eq "")
{
   die ("ERROR: You must define a master server in $configfile\n");
}

if ($varhash{"rsyncrepname"} eq "")
{
   die ("ERROR: You must define a rsync repository in $configfile\n");
}
 
# End of var reading from config file


# Checking folder for pid file and named pipes.

if (!(-d $varhash{varrundir}))
{
   mkdir ($varhash{varrundir},500);
}


# Guessing OS

if ($^O =~ /AIX/i) 
{
   $os = "AIX"; 
}
elsif ($^O =~ /Linux/i) 
{
   $os = "LINUX"; 

   open (DMSG,"/bin/dmesg |");
   while (<DMSG>)
   {
      chomp($_);

      if ($_ =~ /(hd|sd)([a-z]):.*(cd-r|dvd-r|cdrom).*/i)
      {
         my $cdrom = $1.$2;

         # This disk is a cdrom

         $cdroms{$cdrom} = 1;
      }
   }
   close (DMSG);

   if (-f "/proc/diskstats")
   {
      if (open (DISKSTATS,"< /proc/diskstats"))
      {
         while (<DISKSTATS>)
         {
            chomp($_);

            if ($_ =~ /(hd|sd)([a-z])\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/)
            {
               my $disk = $1.$2;

               if (!exists($cdroms{$disk}))
               {
                  $linuxvaliddisks{$disk} = 1;
               }

            }
         }
         close (DISKSTATS);
      }
   }


}
else
{
   print STDERR "Sorry, not ready yet for $^O, check for new versions\n" ;
   exit;
}

if ($varhash{debug})
{
   print "found OS $os\n";
}

defined ($varhash{$os."mon"}) || die "Not defined monitor stanza for OS $os\n";

$monitor = $varhash{$os."mon"};

defined ($varhash{$varhash{$os."mon"}}) || die "Not defined path for monitor $monitor\n";

$monitorpath = $varhash{$varhash{$os."mon"}};

if ($varhash{debug})
{
   print "Monitorpath is $monitorpath\n";
}

system ("type $monitorpath > /dev/null");

if ($? != 0)
{
   die "Cannot find $monitorpath\n";
}

# Checking for rsync

system ("type rsync > /dev/null");

if ($? == 0)
{
   $rsync = "rsync";
}
elsif (-f "/usr/bin/rsync")
{
   $rsync = "/usr/bin/rsync"; 
}
elsif (-f "/usr/local/bin/rsync")
{
   $rsync = "/usr/local/bin/rsync"; 
}
elsif (-f "/home/remeye/bin/rsync")
{
   $rsync = "/home/remeye/bin/rsync";
}
else
{
   die "Could not find rsync\n";
}

if ($varhash{test} == 0)
{
   # Active pid control

   if (-f $pidfile)
   {
      open (PIDFILE,"< $pidfile");   
      while (<PIDFILE>)
      {
         if ($_ =~ /^(\d+)$/)
         {
            $pid = $1; 
            $pidcheckrequired=1;
         }
      }
      close (PIDFILE);
   }

   if ($pidcheckrequired)
   {
      my $psstring = "";
      my $progname = "";

      if ($0 =~ /([^\/]+)$/)
      {
         $progname = $1;
      }

      if ($varhash{debug})
      {
         print "progname is $progname\n";
      }

      if ($os eq "AIX")
      {
         $psstring = "ps -ef"; 
      }
      else
      {
         $psstring = "ps -weff"; 
      }

      open (PS,"$psstring |");
      while (<PS>)
      {
         if ($_ =~ /^\s*\w+\s+(\d+)\s+\w+/) 
         {
            if ($1 eq $pid) 
            {
               if ($_ =~ /$progname/)
               {
                  print STDERR "$progname is already running... leaving current instance!\n";
                  exit 1;
               }
            }
         }
      }
      close (PS);
   }


   # At this point, the program can go on... It is not already running 


   # storing pid at pid control file

   open (PIDFILE,"> $pidfile") || die "could not open pidfile \"$pidfile\"\n";
   print PIDFILE "$$"; 
   close (PIDFILE);

} # end if !test

# No problem up to this point...

# Retrieving hostname

$hostname=`hostname`;
chomp($hostname);


#TODO
# Add the posibility of different masters if function of hostname
#ENDTODO

# Cheking if data storage dir exists, alternatively it gets created

if (!(-d $varhash{outdir}))
{
   mkdir ($varhash{outdir},500);

   (-d $varhash{outdir}) || die "Could not create dir \"$varhash{outdir}\"\n";
}
else
{
   &rsyncer
}

# Infinite loop, only getting out by killing it or with stop flag

while (! $hastodie)
{
   # VAR
   my $time     = 0;
   my $avm      = 0;
   # ENDVR


   if ($varhash{test} == 0)
   {

      # Checking if there where previous pipes, if positive they get deleted
 
      opendir (VARRUNDIR,"$varhash{varrundir}") || die "Error, cannot open dir \"$varhash{varrundir}\"\n";

      my $fileaux = "";
   
      while (defined ($fileaux = readdir(VARRUNDIR)))
      {
         if ($fileaux =~ /$varhash{fifopostfix}/)
         {
            unlink ($varhash{varrundir}."/".$fileaux); 
            print "Deleting pipe $fileaux\n";
         }
      }
      closedir (VARRUNDIR);

      # Creating the pipe and making monitor write to it

      -p $fifo && unlink ("$fifo");

      system ("mknod $fifo p");

      if ($monitor eq "nmon")
      {
         system ("$monitorpath -F $fifo  -s 20 -c 40 &");
      }
      elsif ($monitor eq "lmon")
      {
         system ("$monitorpath > $fifo &");
      }
      else
      {
         die "Monitor program not defined\n";
      }

      open (FIFO,"< $fifo");

   }
   else
   {
      open (FIFO,"<&STDIN");

   }

   if ($os eq "AIX")
   {
      open (VMSTAT,"vmstat |"); # Getting avm from vmstat

      while (<VMSTAT>)
      {
         chomp($_);

         if ($_ =~ /^(\s*)(\d+)(\s+)(\d+)(\s+)(\d+)/)
         {
            $avm = $6*4/1024;
         }
      }
      close (VMSTAT);
   }

   while (<FIFO>)
   {
      $time = time();
    
      chomp();

      if ($_ !~ /T0001/)
      {
         if ($_ =~ /^DISKREAD,[^,]*,(fd|hd|sd|hdisk|vpath)(.*)/i)
         {
            #Construct the hash map
            @diskreadarray = split(/,/,$1.$2);
         }
         elsif ($_ =~ /^DISKWRITE,[^,]*,(fd|hd|sd|hdisk|vpath)(.*)/i)
         {
            #Construct the hash map
             @diskwritearray = split(/,/,$1.$2);
         }
         if ($_ =~ /^DISKREAD,[^,]*,(\d.*)/)
         {
            my @dataarray = split(/,/,$1);
            my $index = 0;

            my @string = split (/,/,$_);

            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }

            for ($index=0;$index < @diskreadarray;$index++)
            {
               $diskreadhash{$diskreadarray[$index]} = $dataarray[$index];
            }

         }
         elsif ($_ =~ /^DISKWRITE,[^,]*,(\d.*)/)
         {
            #DISKREAD,Disk Read kb/s betenoire,fd0,hda,hda1,hda2,hda3,hda5,hda6
            #DISKWRITE,Disk Write kb/s betenoire,fd0,hda,hda1,hda2,hda3,hda5,hda6

            my @dataarray = split(/,/,$1);
            my $index = 0;
            my $disk  = "";
            my $totalread  = 0;
            my $totalwrite = 0;

            my @string = split (/,/,$_);

            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }

            for ($index=0;$index < @diskwritearray;$index++)
            {
               $diskwritehash{$diskwritearray[$index]} = $dataarray[$index];
            }


            foreach $disk (sort keys %diskwritehash)
            {
               if (($os eq "AIX") || ($os eq "LINUX") && (exists $linuxvaliddisks{$disk}))
               {
                  $totalread  += $diskreadhash{$disk};
                  $totalwrite += $diskwritehash{$disk};
               }
            }

            sendline("DISKREADWRITE,$time,$totalread,$totalwrite\n");

            foreach $disk (sort keys %diskwritehash)
            {
               if (($os eq "AIX") || ($os eq "LINUX") && (exists $linuxvaliddisks{$disk}))
               {
                  sendline("IX,DISKREADWRITE___$disk,$time,$diskreadhash{$disk},$diskwritehash{$disk}\n");
               }
            }

         }
         elsif ($_ =~ /^CPU_ALL,\w+,(\d+\.*\d*)/)
         {
            my @string = split(/,|\s+/,$_);
   
            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }
            my $cpu_user  = $string[2];
            my $cpu_sys   = $string[3];
            my $cpu_wait  = $string[4];
            my $cpu_idle  = $string[5]; 
            my $cpu_total = 100.0; 
            $cpunumber = $string[7];
   
            #open (FILE,$filename);
            #print FILE "CPU,$time,$cpu_user,$cpu_sys,$cpu_wait,$cpu_idle,$cpu_total\n";
            #close (FILE);

            sendline("CPU,$time,$cpu_user,$cpu_sys,$cpu_wait,$cpu_idle,$cpu_total\n");
         }
         elsif ($_ =~ /^CPU(\d+),\w+,(\d+\.*\d*)/)
         {
            my $cpu_index = $1;
   
            my @string = split(/,|\s+/,$_);
   
            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }
            
            my $cpu_user  = $string[2];
            my $cpu_sys   = $string[3];
            my $cpu_wait  = $string[4];
            my $cpu_idle  = $string[5]; 
            my $cpu_max   = 100.0;

            sendline("IX,CPU___$cpu_index,$time,$cpu_user,$cpu_sys,$cpu_wait,$cpu_idle,$cpu_max\n");

            my $cpu_total = $cpu_user + $cpu_sys + $cpu_wait;
   
            if  ($cpu_index == 1)
            {
               $cpu_max = $cpu_total;
               $cpu_min = $cpu_total;
            }
            else
            {
               if ($cpu_total < $cpu_min)
               {
                  $cpu_min = $cpu_total;
               }
               elsif ($cpu_total > $cpu_max)
               {
                  $cpu_max = $cpu_total;
               }
   
            }
            if ($cpu_index == $cpunumber)
            {
               my $smp_ratio = $cpu_max - $cpu_min;

               #open (FILE,$filename);
               #print FILE "SMPRATIO,$time,$smp_ratio,0,100\n";
               #close (FILE);

               sendline ("SMPRATIO,$time,$smp_ratio,0,100\n");

               $cpu_max = 0;
               $cpu_min = 0;
            }
         }
         elsif ($_ =~ /^MEM,\w+,(\d+\.*\d*)/)
         {
            my @string = split(/,|\s+/,$_);
   
            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }
            my $mem_realfreepercent    = $string[2];
            my $mem_virtualfreepercent = $string[3];
            my $mem_realfreemb         = $string[4];
            my $mem_virtualfreemb      = $string[5];
            my $mem_realtotalmb        = $string[6];
            my $mem_virtualtotalmb     = $string[7];
   
            if ($os ne "AIX") 
            {
               $avm = $string[8];
            }
            
            #open (FILE,$filename);
            #print FILE "MEMPERCENT,$time,$mem_realfreepercent,$mem_virtualfreepercent\n";
            #print FILE "AVMEMMB,$time,$mem_realfreemb,$mem_virtualfreemb,$mem_realtotalmb,$mem_virtualtotalmb,$avm\n";
            #close (FILE);

            sendline("MEMPERCENT,$time,$mem_realfreepercent,$mem_virtualfreepercent\nAVMEMMB,$time,$mem_realfreemb,$mem_virtualfreemb,$mem_realtotalmb,$mem_virtualtotalmb,$avm\n");
         }
         elsif ($_ =~ /^PAGE,\w+,(\d+\.*\d*)/)
         {
            my @string = split(/,|\s+/,$_);
   
            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }
            my $page_faults    = $string[2];
            my $page_pgin      = $string[3];
            my $page_pgout     = $string[4];
            my $page_pgsin     = $string[5];
            my $page_pgsout    = $string[6];
            my $page_reclaims  = $string[7];
            my $page_scans     = $string[8];
            my $page_cycles    = $string[9];

            #open (FILE,$filename);
            #print FILE "PAGEINOUT,$time,$page_pgin,$page_pgout,$page_pgsin,$page_pgsout\n";
            #close (FILE);

            sendline("PAGEINOUT,$time,$page_pgin,$page_pgout,$page_pgsin,$page_pgsout\n");
         }
         elsif ($_ =~ /^PROC,\w+,(\d+\.*\d*)/)
         {
            my @string = split(/,|\s+/,$_);
   
            if ($string[1] =~ /^\d+$/)
            {
               $time = $string[1];
            }
            my $proc_runnable    = $string[2]; 
            my $proc_swapin      = $string[3];
            my $proc_pswitch     = $string[4];
            my $proc_syscall     = $string[5];
            my $proc_read        = $string[6];
            my $proc_write       = $string[7];
            my $proc_fork        = $string[8];
            my $proc_exec        = $string[9];
            my $proc_rcvint      = $string[10];
            my $proc_xmtint      = $string[11];
            my $proc_sem         = $string[12];
            my $proc_msg         = $string[13];
   
            #open (FILE,$filename);
            #print FILE "PROCINFO,$time,$proc_runnable,$proc_swapin,$cpunumber\n"; # Sending number of cpus collected in CPU_ALL in order to compare proc_runnable
            #close (FILE);

            sendline("PROCINFO,$time,$proc_runnable,$proc_swapin,$cpunumber\n");

         }
         elsif ($_ =~ /^NET,Network([^,]+),(.+)$/)
         {
            # VAR
            my @input     = split (/,/,$2);
            my $index     = 0;
            # ENDVAR
   
   
            for ($index = 0;$index <=$#input;$index++)
            {
               $interfacehash{$input[$index]} = $index;
            }
         
            
         }
         elsif ($_ =~ /^NET,(\w+),(.+)$/)
         {
            # VAR
            my @input     = split (/,/,$2);
            my $index     = 0;
            my $maybetime = $1;
            my $line      = "";
            # ENDVAR
   
            if ($maybetime =~ /^\d+$/)
            {
               $time = $maybetime;
            }
   
            #open (FILE,$filename);

            # Filling physical if reads

            #print FILE "NETINFOPHYSICALREAD,$time";

            $line = "NETINFOPHYSICALREAD,$time";

            for ($index = 0;$index <=$#physicalifacesread;$index++)
            {
   
               if (defined ($interfacehash{$physicalifacesread[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$physicalifacesread[$index]}]";
                   $line = $line.",$input[$interfacehash{$physicalifacesread[$index]}]";
               }
               elsif (defined ($interfacehash{$linuxphysicalifacesread[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$linuxphysicalifacesread[$index]}]";
                  $line = $line.",$input[$interfacehash{$linuxphysicalifacesread[$index]}]";
               }
               else
               {
                  #print FILE ",0";
                  $line = $line.",0"; 
               }
            }
            #print FILE "\n"; 
            $line = $line."\n";            

            sendline ($line);


            # Filling virtual if reads
            #print FILE "NETINFOVIRTUALREAD,$time";

            $line = "NETINFOVIRTUALREAD,$time";

            for ($index = 0;$index <=$#virtualifacesread;$index++)
            {
               if (defined ($interfacehash{$virtualifacesread[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$virtualifacesread[$index]}]";
                  $line = $line.",$input[$interfacehash{$virtualifacesread[$index]}]";
               }
               elsif (defined ($interfacehash{$linuxvirtualifacesread[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$linuxvirtualifacesread[$index]}]";
                  $line=$line.",$input[$interfacehash{$linuxvirtualifacesread[$index]}]";
               }
               else
               {
                  #print FILE ",0";
                  $line=$line.",0";
               }
            }
             
            #print FILE "\n";
            $line=$line."\n";
 
            sendline($line);


            # Filling physical if writes
            #print FILE "NETINFOPHYSICALWRITE,$time";
            $line="NETINFOPHYSICALWRITE,$time";

            for ($index = 0;$index <=$#physicalifaceswrite;$index++)
            {
               if (defined ($interfacehash{$physicalifaceswrite[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$physicalifaceswrite[$index]}]";
                  $line=$line.",$input[$interfacehash{$physicalifaceswrite[$index]}]";
               }
               elsif (defined ($interfacehash{$linuxphysicalifaceswrite[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$linuxphysicalifaceswrite[$index]}]";
                  $line=$line.",$input[$interfacehash{$linuxphysicalifaceswrite[$index]}]";
               }
               else
               {
                  #print FILE ",0";
                  $line=$line.",0";
               }
            }
            #print FILE "\n"; 
            $line=$line."\n"; 

            sendline($line);
   

            # Filling virtual if writes
            #print FILE "NETINFOVIRTUALWRITE,$time";
            $line="NETINFOVIRTUALWRITE,$time";
           
            for ($index = 0;$index <=$#virtualifaceswrite;$index++)
            {
               if (defined ($interfacehash{$virtualifaceswrite[$index]}))
               {
                  #print FILE ",$input[$interfacehash{$virtualifaceswrite[$index]}]";
                  $line=$line.",$input[$interfacehash{$virtualifaceswrite[$index]}]";
               }
               else
               {
                  #print FILE ",0";
                  $line=$line.",0";
               }
            }
            #print FILE "\n";
            $line=$line."\n";
   
            sendline($line);
            #close (FILE);

            my $key       = "";
   
            foreach $key   (keys %states)
            {
               $states{$key} = 0;
            }
   
            open(NETSTAT,"netstat -an |");
   
            while (<NETSTAT>)
            {
               if ($_ =~ /^(tcp|udp)(.+)\s([^\s]+)\s*$/)
               {
                  if (defined ($states{$3}))
                  {
                     $states{$3}++ ;
                  }
               }
            }
            close (NETSTAT);

            #open (FILE,$filename);
            #print FILE "NETSTAT,$time";
            $line = "NETSTAT,$time";

            foreach $key   (sort keys %states)
            {
               #print FILE ",$states{$key}";
               $line=$line.",$states{$key}";
            }
            #print FILE "\n";
            $line=$line."\n";
            #close (FILE);
            sendline($line);
   
   
         }
      }   
   }
   close(FIFO);

   &rsyncer;
   
   # Comprobamos si se ha de salir
 
   if (-f $varhash{hastodiefile})
   {
      $hastodie = 1;   
      system("rm $varhash{hastodiefile}");
   }
   elsif ($varhash{test})
   {
      $hastodie = 1;   
   }
   
}

exit 0;

sub rsyncer
{
   # VAR
   my $rescode     = 0;
   my $fileaux     = "";
   my %fileauxhash = ();
   my $keyaux      = "";
   # ENDVAR

   if ($varhash{test} == 0)
   {
      opendir (BASEDIR,"$varhash{outdir}");

      while (defined ($fileaux = readdir (BASEDIR)))
      {
         if ($fileaux =~ /($varhash{fileprefix})_([^\.]+)\.(\d+)$/)
         {
            $fileauxhash{$3} = $varhash{outdir}."/".$fileaux; 
         }
      }
      closedir (BASEDIR);

      # Entries are processed in order
      foreach $keyaux (sort {$a <=> $b} keys %fileauxhash)
      {
         system ("$rsync -a $portstring $fileauxhash{$keyaux} $varhash{master}\:\:$varhash{rsyncrepname}"."/stats/in/");

         if ($? == 0)
         {
           # If an entrie has been sent to central repository, it gets erased
           unlink ($fileauxhash{$keyaux});
         }
      }

   } #end if $varhash{test} == 0
}

sub sendline
{
   #VAR
   my $stringtosend = $_[0];
   my $time         = 0;
   #ENDVAR

   if ($sentlines == 0) 
   {
      if ($varhash{test} == 0)
      {
         # Creating filename, with timestamp
         $time = time();
         $filename=">> ".$varhash{outdir}."/".$varhash{fileprefix}."_".$hostname.".".$time;
      }
      else
      {
         $filename = ">&STDOUT";
      }
   }

   open (FILE,$filename);
   print FILE "$stringtosend";
   close (FILE);

   $sentlines++;

   if ($varhash{debug})
   {
      print "Sentlines $sentlines\n";
   }

   if ($sentlines >= $varhash{maxsentlines})
   {
      rsyncer();    
      $sentlines = 0;
   }
}
