#! /usr/bin/perl

use strict;


#########################################################
#
# (c) Serxio Barral - 2005
#
# sumonagent.pl
#
# This perl program is a daemon that calls other 
# data collection programs
#
# It reads the configuration file
#
# /etc/sumon/agent/sumonagent.conf
#
#
#  This program is free software published under the
#  terms of the GNU General Public License.
#
#  For the entire license text please refer to
#  http://www.gnu.org/licenses/gpl.html
#
#  Serial: __5
#
#########################################################


my $confdir    = "/etc/sumon/agent";
my $configfile = "$confdir/sumonagent.conf";

   my %varhash    = (
                       "datadir"          => "/var/sumon/data",
                       "bindir"           => "/usr/local/bin",
                       "master"           => "",
                       "rsyncrepname"     => "sumonData",
                       "checkinterval"    => 10,
                       "sendinterval"     => 900,
                       "changeagent"      => 1,
                       "statsagent"       => 1,
                       "debug" => 0
                    );

open (CONFIGFILE,"< $configfile") || die "could not open config file \"$configfile\"\n";

while (<CONFIGFILE>)
{
   chomp($_);

   if (($_ !~ /^\s*#/) && ($_ =~ /^\s*(\S+)\s*=\s*(\S+)/))
   {
      my $readvar   = $1;
      my $readvalue = $2;


      if (defined ($varhash{$readvar}))
      {
         $varhash{$readvar} = $readvalue;
         if ($varhash{debug})
         {
            print "read from configfile $readvar $readvalue\n";
         }
      }
      else
      {
         print STDERR "variable not known to program: $readvar in config file, maybe an error\n";
      }
   }

}

(-d $varhash{datadir}) || system("mkdir -p $varhash{datadir}");

if ($varhash{"master"} eq "")
{
   die "ERROR: You don't have configured your master server in $configfile\n";
}


if (fork())
{
   # Parent, this will check for processes

   while (1 == 1)
   {
      if (open (PS,"ps -ef |"))
      {
         my @procs = <PS>;

         if ($varhash{"changeagent"})
         {
            if ($varhash{"debug"})
            {
               print "seeking for changeagent\n";
            }

            my @found = grep { $_ =~ /changeagent/} @procs;

            if (@found == 0)
            {
               if ($varhash{"debug"})
               {
                  print "Starting changeagent\n";
               }
               system ($varhash{"bindir"}."/changeagent.pl");
            } 
         }

         if ($varhash{"statsagent"})
         {
            if ($varhash{"debug"})
            {
               print "seeking for changeagent\n";
            }

            my @found = grep { $_ =~ /statsagent/} @procs;

            if (@found == 0)
            {
               if ($varhash{"debug"})
               {
                  print "Starting statsagent\n";
               }
               system ($varhash{"bindir"}."/statsagent.pl");
            } 
         }

         close(PS);
      }
      
      if ($varhash{"debug"})
      {
         print "sleeping for ".$varhash{"checkinterval"}." secs\n";
      }
      sleep ($varhash{"checkinterval"});
   }
}
else
{
   #Child, this one will rsync the data repository

   my $rsync = "";
   my $datadirok = $varhash{"datadir"};

   if ($varhash{"datadir"} !~ /\/$/) # Check if it has an ending slash
   {
      $datadirok .= "/";
   }

   # Checking for rsync

   system ("type rsync > /dev/null");

   if ($? == 0)
   {
      $rsync = "rsync";
   }
   elsif (-f "/usr/bin/rsync")
   {
      $rsync = "/usr/bin/rsync";
   }
   elsif (-f "/usr/local/bin/rsync")
   {
      $rsync = "/usr/local/bin/rsync";
   }
   elsif (-f "/home/remeye/bin/rsync")
   {
      $rsync = "/home/remeye/bin/rsync";
   }
   else
   {
      die "Could not find rsync\n";
   }

   my $rsyncstring = $rsync." -vua ".$datadirok." ".$varhash{"master"}."::".$varhash{"rsyncrepname"}."/data";

   while (1 == 1)
   {
 
      system ("$rsyncstring");
      system ("echo $rsyncstring");

      sleep ($varhash{"sendinterval"});
   }
}

