#! /bin/sh

###########################################
#
# (c) Serxio Barral 2006
#
# Very simple installer for SUMon files
#
# __Serial# 16
#
###########################################

. ./lib/installhelper.sh

OLDBASEDIR="/var/sumon"
OLDBINDIR="/usr/local/bin"
OLDSTATDIR="/var/sumon/stats"
BINDIR="/opt/sumon/server/bin"
WWWDIR="/var/sumon/www"
VARDIR="/var/sumon/server"
ETCDIR="/etc/sumon/server"
RUNDIR="/var/sumon/run"
BASEDIR="${VARDIR}"
STATDIR="${VARDIR}/stats"
DATADIR="${VARDIR}/data"
RRDDIR="${STATDIR}/rrdb"
STATINDIR="${STATDIR}/in"
DATE=`date +"%Y%m%d_%H%M%S"`

CONFIGFILES="masterstats.conf stats.def graphstats.conf"
BINFILES="graphstats.pl masterstats.pl chmgmtinfobuilder.pl difffile.pl showfile.pl"



for file in ${BINFILES}
do
   ps -ef | grep -v grep | grep -i ${file} > /dev/null 2>&1  && {
      echo ""
      echo "It seems that you have the server running (${file}), please stop it and try again."
      echo ""
      exit 10
   }
done



export OVERWRITECONFIGFILES="TRUE"

while getopts "cd" OPTION
do	case "$OPTION" in
	c) OVERWRITECONFIGFILES="FALSE";;
        *) ;;
	esac
done



for dir in ${BINDIR} ${VARDIR} ${ETCDIR} ${RUNDIR} ${STATDIR} ${WWWDIR} ${RRDDIR} ${STATINDIR}
do
   [ -d ${dir} ] || {
      mkdir -p ${dir}
   }
done





# Erasing older versions (< 0.7.0)

[ -d ${OLDSTATDIR}/in ] && {
   mv ${OLDSTATDIR}/in        ${STATDIR}
}

[ -d ${OLDSTATDIR}/discarded ] && {
   mv ${OLDSTATDIR}/discarded ${STATDIR}
}

[ -d ${OLDSTATDIR}/processed ] && {
   mv ${OLDSTATDIR}/processed ${STATDIR}
}

[ -d ${OLDSTATDIR}/rrdb ] && {
   mv ${OLDSTATDIR}/rrdb      ${STATDIR}
}

[ -d ${OLDBASEDIR}/data ] && {
   mv ${OLDBASEDIR}/data      ${BASEDIR}
}

rmdir ${OLDTSTATDIR} > /dev/null 2>&1


for file in ${BINFILES}
do
   FILE="${OLDBINDIR}/${file}"

   [ -f ${FILE} ] && rm ${FILE}
done





# Copying newer files

for file in ${CONFIGFILES}
do
   CONFIGFILE=${ETCDIR}/${file}

   [ -f ${CONFIGFILE} ] && {
      cp ${CONFIGFILE} ${CONFIGFILE}.${DATE}
      echo
      if [ ${OVERWRITECONFIGFILES} == "TRUE" ]
      then
         echo "WARNING*** copied and replaced file ${CONFIGFILE} to $CONFIGFILE.${DATE}"
         echo "please, if you made any change to this file you should replicate them in the new file"
         echo
      fi
   }
done

for file in ${BINFILES}
do
   cp ./server/bin/${file} ${BINDIR}
done

for file in ${CONFIGFILES}
do
   cp ./server/etc/${file} ${ETCDIR}
done

cp -pr ./server/www/*   ${WWWDIR}

if [ ${OVERWRITECONFIGFILES} == "FALSE" ]
then
   for file in ${CONFIGFILES}
   do
      CONFIGFILE=${ETCDIR}/${file}

      [ -f ${CONFIGFILE}.${DATE} ] && {
         cp ${CONFIGFILE}.${DATE} ${CONFIGFILE}
      }
   done
fi

RSYNCFOUND=TRUE
checkForRsync || {
   displayRsyncMessageforServer
   RSYNCFOUND=FALSE
}

checkForRRDTOOL

[ ${RSYNCFOUND} == "TRUE" ] && {
   if [ -f /etc/rsyncd.conf ]
   then
      grep -i "\[Sumon\]" /etc/rsyncd.conf > /dev/null 2>&1
      if [ $? -eq 0 ]
      then
          grep -i path /etc/rsyncd.conf | grep -i "/var/sumon/server" > /dev/null 2>&1 || {
            echo ""
            echo ""
            echo "Mmmm. It seems that you have a Sumon rsync repository, but it is not"
            echo "pointing to /var/sumon/server. Perhaps you come from older versions"
            echo "please, change it and refresh rsync daemon"
            echo ""
            echo ""
          }
      else
         echo ""
         echo ""
         echo "Now you should setup  a rsync repository called Sumon"
         echo "pointing to /var/sumon/server"
         echo ""
         echo ""
      fi
   else
      echo ""
      echo ""
      echo "Mmmm. It seems that you have not configured rsync as a daemon."
      echo "I cannot find a /etc/rsyncd file"
      echo ""
      echo ""
      echo "Once you set it up, you should create a repository called Sumon"
      echo "pointing to /var/sumon/server"
      echo ""
      echo ""
   fi
}


echo
echo
echo "Please, configure your httpserver for serving      "
echo "${WWWDIR} directory                                "
echo 
echo "Also, httpserver user should be able to write to  "
echo "directory /var/sumon/www/graphics                  "
echo
echo
echo "Maybe you want to start the server now by          "
echo "executing ${BINDIR}/masterstats.pl            "
