#! /usr/bin/perl -w

use strict;

#########################################################
#
# (c) Serxio Barral - 2005
#
# GraphStats.pl
#
# This perl program generates performance graphics
#
# It reads two files
#
# $varhash{defsfile}:
#              db structure (DS, RRA...)
#
# $graphconfig:
#              Graphic files to be generated (for
#              each db as many files as definitions
#              will be generated)
#
#
#              A parameter with switch -f will
#              be used as config file.
#
#              A parameter with structure: 
#              -g GRAPH:Name:time will be used for graph
#              selection
#
#              A parameter with switch -d will
#              be used to select graphics directory
#
#  __Serial# 43
#
#
#  This program is free software published under the
#  terms of the GNU General Public License.
#
#  For the entire license text please refer to
#  http://www.gnu.org/licenses/gpl.html
#:
#
#########################################################



#VAR
my $confdir             = "/etc/sumon/server";
my $configfile          = "$confdir/graphstats.conf";
my %varhash             = (
           "statsdir"  => "/var/sumon/server/stats",
           "defsfile"  => "$confdir/stats.def",
           "outdir"    => "/var/sumon/www/graphics",
           "debug"     => 0,
                          );
my $graphconfig         = "";
my %grouphash           = ();
my $line                = 0;
my $groupname           = ""; 
my %graphconfighash     = ();
my $argline             = "";
my $selectedhostname    = "";
my $selectedgraphname   = "";
my $selectedstart       = 0;
my $selectedend         = 0;

# Reading argv

$argline = join (" ",@ARGV);

if ($argline =~ /-f\s*(\S+)\s*/)
{
   # There is an user specified configuration file

   $configfile =$1;
}

open (CONFIGFILE,"< $configfile") || die "could not open config file \"$configfile\"\n";

while (<CONFIGFILE>)
{
   chomp($_);

   if (($_ !~ /^\s*#/) && ($_ =~ /^\s*(\S+)\s*=\s*(\S+)/))
   {
      my $readvar   = $1;
      my $readvalue = $2;


      if (defined ($varhash{$readvar}))
      {
         $varhash{$readvar} = $readvalue;
         if ($varhash{debug})
         {
            print "read from configfile $readvar $readvalue\n";
         }
      }
      else
      {
         print STDERR "variable not known to program: $readvar in config file, maybe an error\n";
      }
   }

}

if ($argline =~ /-h\s*(\S+)\s*/)
{
   # There is an user specified hostname

   $selectedhostname = $1;
}

if ($argline =~ /-s\s*(\d+)\s*/)
{
   # There is an user specified start time

   $selectedstart = $1;
}

if ($argline =~ /-e\s*(\d+)\s*/)
{
   # There is an user specified start time

   $selectedend = $1;
}

if ($argline =~ /-l\s*(\S+)\s*/)
{
   # There is an user specified graphic label

   $selectedgraphname = $1;
}

if ($argline =~ /-d\s*(\S+)\s*/)
{
   # There is an user specified graphic label

   $varhash{aoutdir} = $1;
}

if ($argline =~ /-g\s+(\S+)\s*/)
{
   # There is an user specified graphics configuration file

   $graphconfig = $1;
}

my $rrdbdir             = $varhash{statsdir}."/rrdb";
#ENDVAR



(-d $rrdbdir) || die "could not open dbdir \"$rrdbdir\"\n";

&CheckAndCreateDir ($varhash{outdir});

# Configuration file is read

open (CONFIG,"< $varhash{defsfile}") || die "Could not open definitions file \"$varhash{defsfile}\"";

while (<CONFIG>)
{
   $line++;

   chomp($_);
   if ($_ =~ /^(\s*)GROUP(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)/)
   {
      # Definicion de grupo

      # VAR
      my $groupname = lc($4);
      my $step      = $7;
      # ENDVAR

      if (exists ($grouphash{$groupname}))
      {
         die "$varhash{defsfile} : $line  $groupname group already defined\n";
      }

      $grouphash{$groupname}->{step}=$step;
      $grouphash{$groupname}->{ds}=();
      $grouphash{$groupname}->{rra}=();
   }
   if ($_ =~ /^(\s*)DS(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)/)
   {
      # Data Source definition

      # VAR
      my $groupname = lc($4);
      my $dsname    = $7;
      # ENDVAR

      if (defined ($grouphash{$groupname}->{ds}->[$34]))
      {
         die "ERROR  DS index $34 is defined more than once for $groupname\n";
      }
      else
      {
         $grouphash{$groupname}->{ds}->[$34]->{dsname}      = $dsname;
         $grouphash{$groupname}->{ds}->[$34]->{DST}         = $10;
         $grouphash{$groupname}->{ds}->[$34]->{heartbeat}   = $13;
         $grouphash{$groupname}->{ds}->[$34]->{min}         = $16;
         $grouphash{$groupname}->{ds}->[$34]->{max}         = $19;
         $grouphash{$groupname}->{ds}->[$34]->{color}       = $25;
         $grouphash{$groupname}->{ds}->[$34]->{drawable}    = $28;
         $grouphash{$groupname}->{ds}->[$34]->{linetype}    = $31;
      }
   }
   if ($_ =~ /^(\s*)RRA(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\d+\.*\d*)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)/)
   {
      # RRA definition
 
      # VAR
      my $groupname = lc($4);
      my %temphash  = ();
      # ENDVAR

      $temphash{rraname} = $7;
      $temphash{CF}      = $10;
      $temphash{xff}     = $13;
      $temphash{steps}   = $16;
      $temphash{rows}    = $19;

      push(@{$grouphash{$groupname}->{rra}},\%temphash);
   }
}

close (CONFIG);

# Reviewing all DS's

foreach $groupname (keys %grouphash)
{
   my $index=0;

   for ($index=0;$index<=$#{$grouphash{$groupname}->{ds}};$index++)
   {
      if (!defined ($grouphash{$groupname}->{ds}->[$index]))
      {
         die "ERROR not every DS index is defined for $groupname\n";
      }
   }
}


# processing config to graph
  
if ($graphconfig =~ /^\s*GRAPH:\s*(\w+\s*\w*\s*\w+)\s*:(\d+)/)
{
   $graphconfighash{"$1"} = $2;
}
else
{
   die "You must specify a configuration to graph: GRAPH:name:secstodisplay\n";
}


if (($selectedhostname ne "") && ($selectedgraphname ne ""))
{
   my $filename = $rrdbdir."/rrdb_".$selectedhostname."_".$selectedgraphname.".rrd";

   # just drawing an specific graphic

   if (-f $filename)
   {
      drawGraph($selectedhostname,$selectedgraphname,$filename);
   }
}
else
{

   # Every rrdb is searched, and for every one, a graph is generated

   open (GRAPHDIR,"find $rrdbdir -type f |");

   while (<GRAPHDIR>)
   {
      chomp ($_);
   
      if ($_ =~ /rrdb_([^_]+)_([^.]+)\.rrd$/)
      {
         # VAR
         #my $hostname  =   $1;
         #my $graphname =   $2;
         #my $filename  =   $_;
         # ENDVAR
      }
      drawGraph($1,$2,$_);
   }
   close (GRAPHDIR);
}

sub drawGraph
{
   # VAR
   #my $hostname  =   $1;
   #my $graphname =   $2;
   #my $filename  =   $_;
   # ENDVAR


   my ($hostname,$graphname,$filename) = @_;

   if ($varhash{debug})
   {
      print "drawGraph!\n";
   }

   if ((($selectedhostname eq "") || ($hostname =~ /^$selectedhostname$/i)) && (($selectedgraphname eq "") || ($graphname =~ /^$selectedgraphname$/i)))
   {
      my $graphdbname = "";

      if ($varhash{debug}) 
      {
         print "Graphname is $graphname\n";
      }

      if ($graphname =~  /^(.+)___(.+)$/)
      {
         # it's an indexed graph
         $graphdbname = $1; 
      }
      else
      {
         $graphdbname = $graphname;
      }

      if ($varhash{debug}) 
      {
         print "Graphdbname is $graphdbname\n";
      }

      if (exists ($grouphash{$graphdbname})) #Security check... Group should be in config file
      {
         # VAR
         my $key = "";
         # ENDVAR

         if ($varhash{debug})
         {
            print "there is a db definition for this graph\n";
         } 

         # Iterating for every graph to be drawn
         foreach $key (keys %graphconfighash)
         {
            # VAR
            my $index          = 0;
            my $innergraphname = "";
            my $command        = "";
            my $start          = 0;
            my $endstring      = "";
            # ENDVAR

            if ($selectedstart != 0)
            {
               $start = $selectedstart
            }
            else
            {
               $start = time() - $graphconfighash{$key};
            }

            if ($selectedend != 0)
            {
               $endstring = "--end ".$selectedend;
            }
   
            $innergraphname = $graphname."-".$key;


            # Contructing global part of command to execute
            $command  = "rrdtool graph $varhash{outdir}"."/rrdb_".$hostname."_".$innergraphname.".png \\\n";
            $command  = $command." --height 150 --width 600 --alt-autoscale --imgformat PNG --title \"".$innergraphname." - ".$hostname."\" --start ".$start." ".$endstring." \\\n";
   
            # For each graph, a line/area/stack is drawn for every DS
            # So DS's have to be iterated
    
            for ($index=0;$index<= $#{$grouphash{$graphdbname}->{ds}} ;$index++)
            {
               if ($grouphash{$graphdbname}->{ds}->[$index]->{drawable} eq "Y")
               {
                  my $indexstring="line".$index;
                  $command   =   $command." DEF:".$indexstring."=".$filename.":".$grouphash{$graphdbname}->{ds}->[$index]->{dsname}.":AVERAGE ".$grouphash{$graphdbname}->{ds}->[$index]->{linetype}.":".$indexstring."#".$grouphash{$graphdbname}->{ds}->[$index]->{color}.":\"".$grouphash{$graphdbname}->{ds}->[$index]->{dsname}."\"\\\n";
               }

            }
            if ($varhash{debug})
            {
               print "command: $command\n";
            }

            system ("$command\n");
         }
      } 
   }
} 

sub CheckAndCreateDir
{
   my $dir = $_[0];

   if (!(-d $dir))
   {
      system("mkdir $dir");

      (-d $dir) || die "Cannot create rrdbdir \"$dir\"\n";
   }
}

