#! /usr/bin/perl -w

use strict;

#########################################################
#
# (c) Serxio Barral - 2005
#
# MasterStats.pl
#
# This perl program processes data from agents
#
#
#
#  This program is free software published under the
#  terms of the GNU General Public License.
#
#  For the entire license text please refer to
#  http://www.gnu.org/licenses/gpl.html
#
#  __Serial# 182
#
#########################################################


# Statistics files are processed in sorted fashion
my $confdir = "/etc/sumon/server";

# VAR
my %varhash             = (
           "statsdir"       => "/var/sumon/server/stats", 
           "defsfile"       => "$confdir/stats.def",
           "debug"          => "1",
           "rsyncdconf"     => "/etc/rsyncd.conf",
           "rsyncrepname"     => "Sumon",
           "rsyncreppath"     => "/var/sumon/server/"
                          );

my $rsyncbinaryfound    = 0;
my $rsyncisrunning      = 0;

my $rsynclocation       = "";
my $fileprefix          = "gstats";
my $configfile          = "$confdir/masterstats.conf";
my %grouphash           = ();
my $line                = 0;
my $groupname           = "";
my $fileisgood          = 1;
my $argline             = "";

# Reading argv

$argline = join (" ",@ARGV);

if ($argline =~ /-f\s*(\S+)\s*/)
{
   # There is an use specified  configuration file

   $configfile =$1;
}

open (CONFIGFILE,"< $configfile") || die "could not open config file \"$configfile\"\n";

while (<CONFIGFILE>)
{
   chomp($_);

   if (($_ !~ /^\s*#/) && ($_ =~ /^\s*(\S+)\s*=\s*(\S+)/))
   {
      my $readvar   = $1;
      my $readvalue = $2;


      if (defined ($varhash{$readvar}))
      {
         $varhash{$readvar} = $readvalue;
         if ($varhash{debug})
         {
            print "read from configfile $readvar $readvalue\n";
         }
      }
      else
      {
         print STDERR "variable not known to program: $readvar in config file, maybe an error\n";
      }
   }

}

my $rrdbdir             = $varhash{statsdir}."/rrdb";
my $indir               = $varhash{statsdir}."/in";
my $discardeddir        = $varhash{statsdir}."/discarded";
my $processeddir        = $varhash{statsdir}."/processed";
# ENDVAR

&CheckAndCreateDir ($rrdbdir);
&CheckAndCreateDir ($indir);
&CheckAndCreateDir ($discardeddir);
&CheckAndCreateDir ($processeddir);

# Checking for rsync 


if (system ("type rsync > /dev/null 2>&1") == 0)
{
   $rsyncbinaryfound = 1;
}

if (!$rsyncbinaryfound)
{
   print "WARNING: I have not found rsync at common locations\n";
   print "\n";
   print "maybe you want to check if you have it installed,\n";
   print "as rsync is needed for receiving files from agents\n";
}
else
{
   open (PS,"ps -ef |");
   while (<PS>)
   {
      chomp ($_);   
      
      if ($_ =~ /rsync/)
      {
         if ($_ =~ /--daemon/)
         {
            $rsyncisrunning = 1;
         }
      }
   }
   close (PS);

   if (!$rsyncisrunning)
   {
      print "WARNING: It seems that rsync is not running as a daemon\n";
      print "\n";
      print "rsync should be running as a daemon in order to receive\n";
      print "files from agents\n";
      print "\n";
      print "...and a rsync repository should be configured pointing\n";
      print "to $indir\n";
   }
   else
   {
      if (-f "$varhash{rsyncdconf}")
      {
         my $rsyncrepfound = 0;

         open (RSYNCDCONF,"< $varhash{rsyncdconf}");
         while (<RSYNCDCONF>)
         {
            chomp($_); 
            if ($_ =~ /$varhash{rsyncreppath}/)
            {
               $rsyncrepfound = 1;
            }
         }
         close (RSYNCDCONF);

         if (!$rsyncrepfound)
         {
            print "WARNING, a rsync repository pointing to $indir\n";
            print "has not been found in $varhash{rsyncdconf}\n";
            print "you're probably not receiving data from agents\n";
            print "due to this\n";
         }
      } 
      else
      {
         print "WARNING: It seems that there is no rsyncd.conf file\n";
         print "\n";
         print "you should configure a repository pointing to $indir";
      }
   }
}



# Reading configuration file
open (DEFS,"< $varhash{defsfile}") || die "Cannot open definitions file \"$varhash{defsfile}\"\n";

while (<DEFS>)
{
   $line++;

   chomp($_);

   #if ($_ =~ /^(\s*)INDEXEDGROUP(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)/)
   #{
   #   my $groupname = $4;
   #   my $step      = $7;
   #   
   #   if (exists ($grouphash{$groupname}))
   #   {
   #      die "$varhash{defsfile} : $line Group $groupname already defined\n";
   #   }
   #   
   #   $grouphash{$groupname}->{step} = $step;
   #   $grouphash{$groupname}->{type} = 1; # 0 regular, 1 indexed
   #   $grouphash{$groupname}->{ds}   = ();
   #   $grouphash{$groupname}->{rra}  = ();
   #}
   if ($_ =~ /^(\s*)GROUP(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)/)
   {
      my $groupname = $4;
      my $step      = $7;
      
      if (exists ($grouphash{$groupname}))
      {
         die "$varhash{defsfile} : $line Group $groupname already defined\n";
      }
      
      $grouphash{$groupname}->{step} = $step;
      $grouphash{$groupname}->{type} = 0; # 0 regular, 1 indexed
      $grouphash{$groupname}->{ds}   = ();
      $grouphash{$groupname}->{rra}  = ();
   }
   if ($_ =~ /^(\s*)DS/)
   {
    
      my @string = split(/\s*:\s*/,$_);

      my $groupname = $string[1];
      my $dsname    = $string[2];
      my $index     = $string[7]; 

      if (defined ($grouphash{$groupname}->{ds}->[$index]))
      {
         die "$varhash{defsfile} : $line DS index $index defined more than one time for group $groupname\n";
      }
      else
      {
         $grouphash{$groupname}->{ds}->[$index]->{dsname}      = $dsname;
         $grouphash{$groupname}->{ds}->[$index]->{DST}         = $string[3];
         $grouphash{$groupname}->{ds}->[$index]->{heartbeat}   = $string[4];
         $grouphash{$groupname}->{ds}->[$index]->{min}         = $string[5];
         $grouphash{$groupname}->{ds}->[$index]->{max}         = $string[6];
      }
   }
   if ($_ =~ /^(\s*)RRA(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)(\s*):(\s*)(\d+\.*\d*)(\s*):(\s*)(\w+)(\s*):(\s*)(\w+)/)
   {
      my $groupname = $4;
      my %temphash  = ();

      $temphash{rraname} = $7;
      $temphash{CF}      = $10;
      $temphash{xff}     = $13;
      $temphash{steps}   = $16;
      $temphash{rows}    = $19;

      push(@{$grouphash{$groupname}->{rra}},\%temphash);
   }
}

close (DEFS);

# All DS's are reviewed to be sure that all indexes do exist

foreach $groupname (keys %grouphash)
{
   my $index=0;

   for ($index=0;$index<=$#{$grouphash{$groupname}->{ds}};$index++)
   {
      if (!defined ($grouphash{$groupname}->{ds}->[$index]))
      {
         die "ERROR No estn definidos todos los indices de DS para $groupname\n";
      }
   }
}

my $key;
my $group;

if ($varhash{debug})
{
   foreach $group (keys %grouphash)
   {
      my $ds;

      my $rra;

      print STDERR "$group, type $grouphash{$group}->{type} with $#{$grouphash{$group}->{ds}} +1 DS's \n";

      print STDERR "   step -> $grouphash{$group}->{step} \n";
      
      for ($ds=0;$ds<=$#{$grouphash{$group}->{ds}};$ds++)
      {
         my $subkey;
   
         print STDERR "   DS -> $grouphash{$group}->{ds}->[$ds]->{dsname}\n" ;

         foreach $subkey (keys %{$grouphash{$group}->{ds}->[$ds]})
         {
            print STDERR "      $subkey -> $grouphash{$group}->{ds}->[$ds]->{$subkey}\n"
         }
      }

      for ($rra=0;$rra<=$#{$grouphash{$group}->{rra}};$rra++)
      {
         my $subkey;

         print STDERR "   RRA -> $rra\n";

         foreach $subkey (keys %{$grouphash{$group}->{rra}->[$rra]})
         {
            print STDERR "      $subkey -> $grouphash{$group}->{rra}->[$rra]->{$subkey}\n"
         }
      }
  
   }

}

# Infinite loop, keep processing stats file forever
while ()
{
 
   # Each iteration, new files are processed
   # files are retrieved in a sorted fashion
   # by using timestamp included in file name

   ($varhash{debug}) && print STDERR "Main loop iteration\n";

   open (DIR,"find $indir -type f 2>/dev/null | sort -t. -n  +1  |");

   while (<DIR>)
   {
      my $filename       = "";
      my $filetimestamp  = "";
      my $hostname       = "";
      my $nopathfilename = "";
      

      chomp($_);
 

      if ($_ =~ /\/($fileprefix)_(.+)\.(\d+)/)
      {
         $hostname      = $2;
         $filetimestamp = $3;

         #if ($hostname =~ /esp-ge02/)
         $filename=$indir."/".$fileprefix."_".$hostname.".".$filetimestamp;
         $nopathfilename = $fileprefix."_".$hostname.".".$filetimestamp;
    
         # Got a new file...

        ($varhash{debug}) && print STDERR "Processing file $filename\n";
  
        open (FILE,"< $filename");

        $fileisgood = 1;

        while (<FILE>) 
        {
           if ($fileisgood)
           {
              chomp($_);

              ($varhash{debug}) && print "processed line is:\n $_\n";
   
              if (($_ =~ /^[A-Z_]+\d*,(\d+),\d+\.*\d*/) || ($_ =~ /^IX,[A-Z_]+___.+,(\d+),\d+\.*\d*/))# if this format, then it should be a metrics file 
              {
                 # VAR
                    my @metric    = ();
                    my $header    = "";
                    my $timestamp = "";
                    my $groupname = "";
                 # ENDVAR

                 @metric=split(/,/,$_);


                 if ($metric[0] eq "IX")    # IX is prefix for indexed groups
                 {
                    shift(@metric); #so the handling is the same as for regular groups

                    if ($metric[0] =~ /^([A-Z_]+)___(.+)$/)
                    {
                       $groupname = $1;
                    }
                 }
                 else
                 {
                    $groupname = $metric[0];
                 }

                 $header    = $metric[0];      # CPU, MEM, PROCINFO,... 

                 $timestamp = $metric[1];      # EPOCH
   

                 # If there is a header, we can process it
                 if (exists ($grouphash{$groupname}) && defined ($grouphash{$groupname}))
                 {
                    my $rrdfile = lc($rrdbdir."/RRDB_".$hostname."_".$header.".rrd");

   
                    if (!(-f $rrdfile))
                    {
                       my $command = "";
                       my $index   = 0;
   
                       ($varhash{debug}) && print STDERR "Creating database for $header $hostname...\n";
   
                       $command =  "rrdtool create $rrdfile --start $filetimestamp \\\n";

                       $command = $command . "--step $grouphash{$groupname}->{step} \\\n";
                       
                       # The command is built up with every DS
                        
                       for ($index=0;$index<=$#{$grouphash{$groupname}->{ds}};$index++) 
                       {
                          $command = $command . "DS:" . $grouphash{$groupname}->{ds}->[$index]->{dsname} . ":" . $grouphash{$groupname}->{ds}->[$index]->{DST} . ":" . $grouphash{$groupname}->{ds}->[$index]->{heartbeat} . ":" . $grouphash{$groupname}->{ds}->[$index]->{min} . ":" . $grouphash{$groupname}->{ds}->[$index]->{max} . " \\\n";
                       }
                       # The command is built up with every RRA
   
                       for ($index=0;$index<=$#{$grouphash{$groupname}->{rra}};$index++) 
                       {
                          $command = $command . "RRA:" . $grouphash{$groupname}->{rra}->[$index]->{CF} . ":" . $grouphash{$groupname}->{rra}->[$index]->{xff} . ":" . $grouphash{$groupname}->{rra}->[$index]->{steps} . ":" . $grouphash{$groupname}->{rra}->[$index]->{rows} . " \\\n"; 
                       }
                       
                       ($varhash{debug}) && print STDERR "command: $command\n";
                       system "$command\n"; 
     
                       if ($? != 0)
                       {
                          # Something went wrong!
                          print STDERR "Error creating db for $header $hostname\n";
                       }
                       else
                       {
                          InsertData ($groupname,\@metric,$rrdfile,$filename);
                       }
   
                       # db does not exist, so it is created
                    }
                    else
                    {
                       # The database exists
    
                       if  (InsertData ($groupname,\@metric,$rrdfile,$filename,$nopathfilename) != 0)
                       {
                          $fileisgood = 0;
                       }
                    }
                 }
              }
           }
        }
        close(FILE);
        
        # And last, deleting the file
        if (-f $filename)
        {
           ($varhash{debug}) && system ("echo mv $filename $processeddir");
           system ("mv $filename $processeddir");
        }
      }
   }
   close(DIR);
   sleep (1);
}

sub InsertData
{
   my $metric         = "";
   my $filename       = "";
   my $rrdfile        = "";
   my $nopathfilename = "";
   my $groupname      = "";
   my $timestamp      = "";
   my $index          = 0; 
   my $offset         = 2;
   my $rescode        = 0;
   my $output         = "";

   $groupname      = $_[0];      # CPU, MEM, PROCINFO,...
   $metric         = $_[1];
   $rrdfile        = $_[2]; 
   $filename       = $_[3]; 
   $nopathfilename = $_[4];
   $timestamp      = $metric->[1];      # EPOCH

   if (($#{$metric}) != ($#{$grouphash{$groupname}->{ds}} + $offset))
   {
      print STDERR "Error, parameter number does not match for $rrdfile   found $#{$metric} - 2 and there should be $#{$grouphash{$groupname}->{ds}}\n";
   }
   else
   {
      my $command = "rrdtool update $rrdfile \\\n"."$timestamp";

      for ($index=$offset;$index<=$#{$metric};$index++)
      {
         $command = $command.":"."$metric->[$index]";
      }

      if ($varhash{debug})
      {
         print "command is $command \n";
      }
      $output=`$command 2>&1`;
   
      if (($? != 0) || ($output =~ /error/i))
      { 
         my $mvcommand="mv ".$filename." ".$discardeddir."/".$nopathfilename;

         print STDERR "Error running $command $? : $output\n";
         $rescode = 1;
         ($varhash{debug}) && print "$mvcommand\n";
         system ("$mvcommand\n");
      }

   }
   return $rescode;

 }

sub CheckAndCreateDir
{
   my $dir = $_[0];

   if (!(-d $dir))
   {
      system("mkdir $dir");
  
      (-d $dir) || die "Cannot create rrdbdir \"$dir\"\n";
   }
}
